c
c******************************************************************************
c  MINLBFGS
c******************************************************************************
c
      subroutine minlbfgs(optinp, ind, n, ierr)
c
c     Subroutine minlbfgd
c
c     Created:  Dec. 06, 2006 by Yan Zhang
c     Revised:  Feb. 14, 2008 by Yan Zhang                       
c
c     Revised:
c
c     Purpose: The subroutine minlbfgs is a limited memory BFGS quasi-newton 
c              nonlinear optimization routine. The subroutine is based on the 
c              lbfgs (called by minimize) subroutine in TINKER.  
c
c
c     literature references:
c
c     J. Nocedal, "Updating Quasi-Newton Matrices with Limited
c     Storage", Mathematics of Computation, 35, 773-782 (1980)
c
c     D. C. Lui and J. Nocedal, "On the Limited Memory BFGS Method
c     for Large Scale Optimization", Mathematical Programming,
c     45, 503-528 (1989)
c
c     J. Nocedal and S. J. Wright, "Numerical Optimization",
c     Springer-Verlag New York, 1999, Section 9.1
c
c     Called by:  mlohook
c     Calls:  optghk, search, dispxyzg, disconv 
c
c     Variables defined here:  
c
      use files, only : outfile, moldenfile, molden5                    
      use input, only : mlopt, mlinp, maxatm
      use optimize, only : ener, grad, hess
      
      implicit none
c
c i/o variables
c
      integer            ierr
      type (mlopt)    :: optinp
      integer            n, ind(n)
c
c local variables
c
      integer, parameter :: maxsav=20
      integer i, j, k, m
      integer nerr, maxerr, msav, ncalls, niter
      integer n3tm, max3atm, maxiter, intmax
      integer muse
c
c added by Yan Zhang 02/04/2008
c
      integer ii, jj, kk, id, froznatm, nunf
      double precision frozf(n)
c
c end by Yan Zhang 02/01/2008
c
      logical done

      character*9 blank, status
      double precision rms, ener_old, gamma 
      double precision g_norm, g_rms, scal, fctmin
      double precision maxdx, rmsdx, maxgrd, rmsgrd, g_max, dx_max, tmp
      double precision f_move, x_move, beta, ys, yy 
      double precision stpmin, stpmax, cappa, slpmax, angmax
      double precision g(n), p(n), q(n), r(n), h0(n)
      double precision x(n), x_old(n), g_old(n)
      double precision rho(maxsav), alpha(maxsav), s(n,maxsav)
      double precision y(n,maxsav) 
c
c Parameters
c
      double precision, parameter :: eps = 1.0d-14
      double precision, parameter :: angtoau = 0.529177249d0
      double precision, parameter :: maxstep = 1.0d0
c
c Parameters fo the optimization
c
      maxgrd  = optinp%gcomp
      rmsgrd  = optinp%rmsgrad
      maxdx   = optinp%maxdx
      rmsdx   = optinp%rmsdx
      stpmin  = optinp%minstep
      stpmax  = optinp%maxstep
      cappa   = optinp%cappa
      slpmax  = optinp%maxslope
      angmax  = optinp%maxang
      intmax  = optinp%intpolat
      maxiter = optinp%niter
      fctmin  = optinp%minenr
c
c initial parameters
c
      max3atm = 3*maxatm
      n3tm    = 3*mlinp%geom%natoms

      ierr    = 0
      nerr    = 0
      maxerr  = 2 
      ncalls  = 1
      niter   = 0      
      maxiter = maxiter + 1
      blank   = '       '
      done    = .false.
c
c  Open molden file                                                     
c
      open (unit=moldenfile, status='unknown', file=molden5, err=9993)      	       
c
c set frozen factor
c added by Yan Zhang 02/14/2008
c
      froznatm = optinp%frozenatm     

      do i = 1, n
         frozf(i) = 1.d0
      end do
c
c mult factors of the frozen atom are set to be zero.
c
      do i = 1, froznatm    
         id = optinp%frozeatmid(i)
         do j = -2, 0
            k = 3*id + j  
            do ii = 1, n 
               jj = ind(ii)
               if(k.eq.jj) frozf(ii) = 0.d0
            end do
         end do
      end do
     
      nunf  = 0
      do i = 1, n 
         if(frozf(i).eq.1.d0)  nunf  = nunf + 1
      end do         
c
c end by Yan Zhang 02/14/2008
c
c     set scaling parameter for function and derivative values;
c
      scal=12.d0
c
c     scale the coordinates of each active atom
c
      do i = 1, n
        j = ind(i)
        x(i) = mlinp%geom%aucoord(j) * scal
      end do
c
c
c     initialize some values to be used below
c
      if (n .gt. max3atm) then
         write (outfile,10)
   10    format (/,' LBFGS  --  Too many atoms')
         ierr = 1
         return
      end if
c
c modified by Yan Zhang 02/14/2008
c
c      rms = sqrt(dble(n))
      rms = sqrt(dble(nunf))
c
c     set default parameters for the optimization
c
c      msav = min(n,maxsav)
      msav = min(nunf,maxsav)
c
c end by Yan Zhang 02/14/2008
c     evaluate the function and get the initial gradient
c
      call optghk

      ener_old = ener
      do i=1, n
         j = ind(i)
         g(i) = grad(j)/scal
c
c added by Yan Zhang 02/14/2008
c
         g(i) = g(i)*frozf(i)
c
c added by Yan Zhang 02/14/2008
c
      end do

      m = 0
      gamma  = 1.0d0
      g_norm = 0.0d0
      g_rms  = 0.0d0
      do i = 1, n
         g_norm = g_norm + g(i)*g(i)
         g_rms = g_rms + (g(i)*scal)**2
      end do
      g_norm = sqrt(g_norm)
      g_rms = sqrt(g_rms) / rms
      f_move = 0.5d0 * stpmax * g_norm
c
c     print initial information prior to first iteration
c
      write(outfile,50)
      write(outfile,60)

50    format (/,80('-'),/)
60       format (' Limited Memory BFGS Quasi-Newton',
     &                 ' Optimization :',/)
c
c     write initial intermediate prior to first iteration
c
      call dispxyzg( 0, mlinp%geom, ener, grad(:n3tm))
c
c  Write the geometry to molden file                                    
c
         write (moldenfile,*) mlinp%geom%natoms
         write (moldenfile,9000) 0, ener
         do i=1, mlinp%geom%natoms
            write (moldenfile,9010) mlinp%geom%atm(i), 
     *         mlinp%geom%angcoord(3*i-2), mlinp%geom%angcoord(3*i-1),
     *         mlinp%geom%angcoord(3*i)
         enddo
9000     format ('Iteration: ',i5,2x,'Energy (a.u.): ',f16.8)	 
9010     format (2x,a2,3(3x,f16.8))
c
c     tests of the various termination criteria
c
      if (niter .ge. maxiter) then
        ierr = 1
        write(outfile,*)'Error: the number of the optimized interation.'
        return
      end if

      if (ener .le. fctmin) then
        done = .true.
        status = 'smallfct'
      end if

c
c     start of a new limited memory BFGS iteration
c
      dowhile (.not. done)
         niter = niter + 1
         muse = min(niter-1,msav)
         m = m + 1
         if (m .gt. msav)  m = 1
c
c     estimate Hessian diagonal and compute the Hg product
c
         do i = 1, n
            h0(i) = gamma
c
c added by Yan Zhang 02/14/2008          
c
            h0(i) = h0(i)*frozf(i)
c
c end by Yan Zhang 02/14/2008
c
            q(i) = g(i)
         end do
         k = m
         do j = 1, muse
            k = k - 1
            if (k .eq. 0)  k = msav
            alpha(k) = 0.0d0
            do i = 1, n
               alpha(k) = alpha(k) + s(i,k)*q(i)
            end do
            alpha(k) = alpha(k) * rho(k)
            do i = 1, n
               q(i) = q(i) - alpha(k)*y(i,k)
            end do
         end do
         do i = 1, n
            r(i) = h0(i) * q(i)
         end do
         do j = 1, muse
            beta = 0.0d0
            do i = 1, n
               beta = beta + y(i,k)*r(i)
            end do
            beta = beta * rho(k)
            do i = 1, n
               r(i) = r(i) + s(i,k)*(alpha(k)-beta)
            end do
            k = k + 1
            if (k .gt. msav)  k = 1
         end do
c
c     set search direction and store current point and gradient
c
         do i = 1, n
            p(i) = -r(i)
            x_old(i) = x(i)
            g_old(i) = g(i)
         end do
c
c     perform line search along the new conjugate direction
c
         status = blank
         call search (optinp,n,ind,ener,g,x,p,f_move, 
     *                frozf, ncalls,status)
c
c     update variables based on results of this iteration
c
         ys = 0.0d0
         yy = 0.0d0
         do i = 1, n
            s(i,m) = x(i) - x_old(i)
            y(i,m) = g(i) - g_old(i)
            ys = ys + y(i,m)*s(i,m)
            yy = yy + y(i,m)*y(i,m)
         end do
         gamma = abs(ys/yy)
         rho(m) = 1.0d0 / ys
c
c     get the sizes of the moves made during this iteration
c
         f_move = ener_old - ener
         ener_old = ener

         x_move = 0.0d0
         dx_max = 0.d0
         tmp=0.d0

         do i = 1, n
            x_move = x_move + ((x(i)-x_old(i))/scal)**2
            tmp=abs((x(i)-x_old(i))/scal)
            if(dx_max.le.tmp) dx_max=tmp
         end do
         x_move = sqrt(x_move) / rms
c
c     compute the rms gradient per optimization parameter
c
         g_rms = 0.0d0
         g_max = 0.d0
         tmp   = 0.d0         

         do i = 1, n
            g_rms = g_rms + (g(i)*scal)**2
            tmp=abs(g(i)*scal)
            if(g_max.le.tmp) g_max=tmp
         end do
         g_rms = sqrt(g_rms) / rms
c
c     display current step in optimization
c
      call dispxyzg(niter, mlinp%geom, ener, grad(:n3tm))
      call disconv(g_max, g_rms, dx_max, x_move, maxgrd, rmsgrd,
     $                  maxdx, rmsdx)
c
c  Write the geometry to molden file                                    
c
         write (moldenfile,*) mlinp%geom%natoms
         write (moldenfile,9000) niter, ener
         do i=1, mlinp%geom%natoms
           write (moldenfile,9010) mlinp%geom%atm(i), 
     *         mlinp%geom%angcoord(3*i-2), mlinp%geom%angcoord(3*i-1),
     *         mlinp%geom%angcoord(3*i)
         enddo
c
c     test for error due to line search problems
c
         if (status.eq.'BadIntpln' .or. status.eq.'IntplnErr') then
            nerr = nerr + 1
            if (nerr .ge. maxerr)  done = .true.
         else
            nerr = 0
         end if
c
c     test for too many total iterations
c
         if (niter .ge. maxiter) then
            status = 'iterlimit'
            done = .true.
         end if
c
c     test the normal termination criteria
c
         if (ener .le. fctmin) then
            status = 'smallfct '
            done = .true.
         end if

         if ((g_rms .le. rmsgrd). and. (g_max.le.maxgrd).and.
     $       (dx_max.le.maxdx).and.(x_move.le.rmsdx)) then
            status = 'converg'
            done = .true.
         end if
      end do
c
c  Display final step in optimization and print appropriate exit message.
c
      if (status.eq.'smallfct') then
         write (outfile, 2000)
         call dispxyzg(niter, mlinp%geom, ener, grad(:n3tm))

      elseif (status.eq.'converg') then
         write (outfile, 2100)
         call dispxyzg(niter, mlinp%geom, ener, grad(:n3tm))

      elseif (status.eq.'iterlimit') then
         write (outfile, 2200)
         call dispxyzg(niter, mlinp%geom, ener, grad(:n3tm))
         ierr = 1

      else
         write (outfile, 2300)status         
         call dispxyzg(niter, mlinp%geom, ener, grad(:n3tm))
         ierr = 1
      endif


c
c  Close molden file                                                    
c
      close (moldenfile,err=9993)

2000  format (/,' Optimization Convergence due to Small Energy',/)


2100  format (/80('*'),/28x,'OPTIMIZATION SUCCESSFUL.',
     *          /80('*'),/)

2200  format (/5x,'OPTIMIZATION FAILURE: Max iterations reached.',/)

2300  format (/5x,'OPTIMIZATION FAILURE: Incomplete Convergence due to',
     $            a9)

      return
9993  write (outfile,*) 'Error creating Molden input file'
      stop 'LBFGS'      

      close (moldenfile,err=9993)

      return
      end
c
c******************************************************************************
c  SEARCH
c******************************************************************************
c
      subroutine search(optinp,nvar,ind,f,g,x,p,f_move,
     *                  ff, ncalls,status)
c
c     Subroutine search
c
c     Created:  Dec. 06, 2006 by Yan Zhang
c     Revised:  Feb. 14, 2008 by Yan Zhang
c
c     Purpose: This is a unidimensional line search based upon parabolic 
c              extrapolation and cubic interpolation using both function and
c              gradient values; if forced to search in an uphill direction,
c              return is after the initial step. The subroutine is based on the 
c              search (called by lbfgs) subroutine in TINKER.  
c
c
c     Called by:  minlbfgs
c     Calls:  lbfgsoptg
c
c     Variables defined here:  none.
c
      use files, only : outfile, moldenfile, molden5                    
      use input, only : mlopt, mlinp, maxatm
      use optimize, only : ener, grad, hess
      
      implicit none
c
c i/o variables
c
      type (mlopt)    :: optinp
      integer            nvar, ncalls
      integer            ind(nvar)
      double precision   f, f_move, g(nvar)
      double precision   x(nvar), ff(nvar)
      character*9        status      
c
c local variables
c
      integer i, intpln, intmax

      character*9      blank      
      logical restart

      double precision s_norm, g_norm, cosang, angle
      double precision step, parab, cube, cubstp, sss, ttt
      double precision f_0, f_1, f_a, f_b, f_c, sg_0, sg_1
      double precision sg_a, sg_b, sg_c, scal 
      double precision stpmin, stpmax, cappa, slpmax, angmax
      double precision p(nvar), s(nvar), x_0(nvar)
c
c Parameters
c
      double precision, parameter :: eps = 1.0d-14
      double precision, parameter :: angtoau = 0.529177249d0
      double precision, parameter :: maxstep = 1.0d0
      double precision, parameter :: radian=57.29577951308232088d0       
c
c
c     use default parameters for the line search if needed
c
      stpmin  = optinp%minstep
      stpmax  = optinp%maxstep
      cappa   = optinp%cappa
      slpmax  = optinp%maxslope
      angmax  = optinp%maxang
      intmax  = optinp%intpolat

      blank='      '
      scal=12.d0
c
c     copy the search direction into a new vector
c
      do i = 1, nvar
         s(i) = p(i)
      end do
c
c     compute the length of gradient and search direction
c
      g_norm = 0.0d0
      s_norm = 0.0d0
      do i = 1, nvar
         g_norm = g_norm + g(i)*g(i)
         s_norm = s_norm + s(i)*s(i)
      end do
      g_norm = sqrt(g_norm)
      s_norm = sqrt(s_norm)
c
c     store initial function, then normalize the
c     search vector and find projected gradient
c
      f_0 = f
      sg_0 = 0.0d0
      do i = 1, nvar
         x_0(i) = x(i)
         s(i) = s(i) / s_norm
         sg_0 = sg_0 + s(i)*g(i)
      end do
c
c     check the angle between the search direction
c     and the negative gradient vector
c
      cosang = -sg_0 / g_norm
      cosang = min(1.0d0,max(-1.0d0,cosang))
      angle = radian * acos(cosang)
      if (angle .gt. angmax) then
         status = 'WideAngle'
         return
      end if
c
c     set the initial stepsize to the length of the passed
c     search vector, or based on previous function decrease
c
      step = 2.0d0 * abs(f_move/sg_0)
      step = min(step,s_norm)
      if (step .gt. stpmax)  step = stpmax
      if (step .lt. stpmin)  step = stpmin
c
c     beginning of the parabolic extrapolation procedure
c
   10 continue
      restart = .true.
      intpln = 0
      f_b = f_0
      sg_b = sg_0
c
c     replace last point by latest and take another step
c
   20 continue
      f_a = f_b
      sg_a = sg_b
      do i = 1, nvar
         x(i) = x(i) + step*s(i)
      end do
c
c     get new function and projected gradient following a step
c
      ncalls = ncalls + 1
c
c modified by Yan zhang 02/14/2008
c
c      call lbfgsoptg(nvar, x, ind, scal, f_b, g)
      call lbfgsoptg(nvar, ff, x, ind, scal, f_b, g)
c
c end by Yan Zhang 02/14/2008
c
      sg_b = 0.0d0
      do i = 1, nvar
         sg_b = sg_b + s(i)*g(i)
      end do
c
c     scale stepsize if initial gradient change is too large
c
      if (abs(sg_b/sg_a).ge.slpmax .and. restart) then
         do i = 1, nvar
            x(i) = x_0(i)
         end do
         step = step / 10.0d0
         status = 'ScaleStep'
         goto 10
      end if
      restart = .false.
c
c     return if the gradient is small and function decreases
c
      if (abs(sg_b/sg_0).le.cappa .and. f_b.lt.f_a) then
         f = f_b
         if (status .eq. blank)  status = ' Success '
         return
      end if
c
c     interpolate if gradient changes sign or function increases
c
      if (sg_b*sg_a.lt.0.0d0 .or. f_b.gt.f_a)  goto 30
c
c     if the finite difference curvature is negative double the step;
c     or if  step < parabolic estimate < 4*step  use this estimate,
c     otherwise truncate to step or 4*step, respectively
c
      step = 2.0d0 * step
      if (sg_b .gt. sg_a) then
         parab = (f_a-f_b) / (sg_b-sg_a)
         if (parab .gt. 2.0d0*step)  parab = 2.0d0 * step
         if (parab .lt. 0.5d0*step)  parab = 0.5d0 * step
         step = parab
      end if
      if (step .gt. stpmax)  step = stpmax
      goto 20
c
c     beginning of the cubic interpolation procedure
c
   30 continue
      intpln = intpln + 1
      sss = 3.0d0*(f_b-f_a)/step - sg_a - sg_b
      ttt = sss*sss - sg_a*sg_b
      if (ttt .lt. 0.0d0) then
         f = f_b
         status = 'IntplnErr'
         return
      end if
      ttt = sqrt(ttt)
      cube = step * (sg_b+ttt+sss)/(sg_b-sg_a+2.0d0*ttt)
      if (cube.lt.0.0d0 .or. cube.gt.step) then
         f = f_b
         status = 'IntplnErr'
         return
      end if
      do i = 1, nvar
         x(i) = x(i) - cube*s(i)
      end do
c
c     get new function and gradient, then test for termination
c
      ncalls = ncalls + 1
c
c modified by Yan Zhang 02/14/2008
c
c      call lbfgsoptg(nvar, x, ind, scal, f_c, g)
c
      call lbfgsoptg(nvar, ff, x, ind, scal, f_c, g)
c
c end by Yan Zhang 02/14/2008
c
      sg_c = 0.0d0
      do i = 1, nvar
         sg_c = sg_c + s(i)*g(i)
      end do
      if (abs(sg_c/sg_0) .le. cappa) then
         f = f_c
         if (status .eq. blank)  status = ' Success '
         return
      end if
c
c     get the next pair of bracketing points by replacing one
c     of the current brackets with the interpolated point
c
      if (f_c.le.f_a .or. f_c.le.f_b) then
         cubstp = min(abs(cube),abs(step-cube))
         if (cubstp.ge.stpmin .and. intpln.lt.intmax) then
c
c     if the current brackets have slopes of opposite sign,
c     then substitute the interpolated point for the bracket
c     point with slope of same sign as the interpolated point
c
            if (sg_a*sg_b .lt. 0.0d0) then
               if (sg_a*sg_c .lt. 0.0d0) then
                  f_b = f_c
                  sg_b = sg_c
                  step = step - cube
               else
                  f_a = f_c
                  sg_a = sg_c
                  step = cube
                  do i = 1, nvar
                     x(i) = x(i) + cube*s(i)
                  end do
               end if
c
c     if current brackets have slope of same sign, then replace
c     the far bracket if the interpolated point has a slope of
c     the opposite sign or a lower function value than the near
c     bracket, otherwise replace the far bracket point
c
            else
               if (sg_a*sg_c.lt.0.0d0 .or. f_a.le.f_c) then
                  f_b = f_c
                  sg_b = sg_c
                  step = step - cube
               else
                  f_a = f_c
                  sg_a = sg_c
                  step = cube
                  do i = 1, nvar
                     x(i) = x(i) + cube*s(i)
                  end do
               end if
            end if
            goto 30
         end if
      end if
c
c     interpolation has failed, reset to best current point
c
      f_1 = min(f_a,f_b,f_c)
      if (f_1 .eq. f_a) then
         sg_1 = sg_a
         do i = 1, nvar
            x(i) = x(i) + (cube-step)*s(i)
         end do
      else if (f_1 .eq. f_b) then
         sg_1 = sg_b
         do i = 1, nvar
            x(i) = x(i) + cube*s(i)
         end do
      else if (f_1 .eq. f_c) then
         sg_1 = sg_c
      end if
c
c     try to restart from best point with smaller stepsize
c
      if (f_1 .gt. f_0) then
         ncalls = ncalls + 1
c
c modified by Yan Zhang 02/14/2008
c
c      call lbfgsoptg(nvar, x, ind, scal, f, g)
c
         call lbfgsoptg(nvar, ff, x, ind, scal, f, g)
c
c end by Yan Zhang 02/14/2008
c
         status = 'IntplnErr'
         return
      end if
      f_0 = f_1
      sg_0 = sg_1
      if (sg_1 .gt. 0.0d0) then
         do i = 1, nvar
            s(i) = -s(i)
         end do
         sg_0 = -sg_1
      end if
      step = max(cube,step-cube) / 10.0d0
      if (step .lt. stpmin)  step = stpmin
c
c     if already restarted once, then return with best point
c
      if (status .eq. ' ReSearch') then
         ncalls = ncalls + 1
c
c modified by Yan Zhang 02/14/2008
c
c      call lbfgsoptg(nvar, x, ind, scal, f, g)
c
         call lbfgsoptg(nvar, ff, x, ind, scal, f, g)
c
c end by Yan Zhang 02/14/2008
c
         status = 'BadIntpln'
         return
      else
         status = ' ReSearch'
         goto 10
      end if
      end
c
c******************************************************************************
c  LBFGSOPTG
c******************************************************************************
c
      subroutine lbfgsoptg(n, ff, xx, ind, scal, enr, g)
c
c     Subroutine lbfgsoptg
c
c     Created:  Dec. 08, 2006 by Yan Zhang
c     Revised:  Feb. 14, 2008 by Yan Zhang 
c               
c
c     Purpose:  To computes the energy and gradient for a low storage
C               BFGS optimization in Cartesian coordinate space
c        
c     
c     Called by:  search
c     Calls:  optghk
c
c     Variables defined here:  
      use optimize, only : ener, grad
      use input, only : mlinp
      use energy
      use gradient
      use caltyp, only : tgrad
 
      implicit none
c
c i/o variables
c
      integer  n, ind(n)
      double precision xx(n), ff(n), g(n), enr, scal
      double precision, parameter :: angtoau = 0.529177249d0

c
c local variables
c
      integer i, j
c
c
c     translate optimization parameters to atomic coordinates
c
      do i = 1, n
        j = ind(i)
        mlinp%geom%aucoord(j) = xx(i)/scal
      end do
      mlinp%geom%angcoord = mlinp%geom%aucoord*angtoau
c
c     compute and store the energy and gradient
c
      call optghk
      enr = ener
c
c     store Cartesian gradient as optimization gradient
c

      do i = 1, n
        j = ind(i)
        g(i)=grad(j)/scal
c
c added by Yan Zhang 02/14/2008
c
        g(i) = g(i)*ff(i)
c
c end by Yan Zhang 02/14/2008
c
      end do

      return
      end
c
c******************************************************************************
c  DISXYZG
c******************************************************************************
c
      subroutine dispxyzg( iter, geom, ener, grad)
c
c     Subroutine dispxyzg
c
c     Created:  Dec. 8, 2006 by Yan Zhang
c     Revised:
c     Purpose:  Display all information for the current optimization step 
c               in the minlbfgs optimization.
c
c     Called by: minlbfgs 
c     Calls:  
c
c     Variables defined here:  none.
c
      use files, only : outfile
      use input, only : cartgeom

      implicit none

      type (cartgeom) :: geom

      integer iter, i
      double precision  ener, grad(3*geom%natoms)

      write(outfile,1000)  iter
      write(outfile,1150)  ener
      write(outfile,1200)
      write(outfile,1250)  grad
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500)  geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500)  geom%atm(i), geom%angcoord(3*i-2),
     *      geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue
      write(outfile,*)

1000  format(/2x,14('-'),/2x,'Iteration: ',i3,/2x,14('-')/)
1150  format(/2x,'Energy:',t30,f19.12)
1200  format(/2x,'Cartesian Gradient:')
1250  format(t20,d15.8,t40,d15.8,t60,d15.8)
1450  format(//2x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//2x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  
      return 
	end
c
c******************************************************************************
c  DISPCONV
c******************************************************************************
c
      subroutine disconv(maxg, rmsg, maxdx, rmsdx, maxgtol, rmsgtol,
     $                  maxdxtol, rmsdxtol)
c
c     Subroutine disconv
c
c     Created:  Dec. 8, 2006 by Yan Zhang
c     Revised:
c     Purpose:  Display all convergence information for the current optimization step 
c               in the minlbfgs optimization.
c
c     Called by: minlbfgs  
c     Calls:  
c
c     Variables defined here:  none.
c
      use files, only : outfile

      implicit none
c      
c I/O variables
c
      double precision  maxg, maxgtol
      double precision  rmsg, rmsgtol
      double precision  maxdx, maxdxtol
      double precision  rmsdx, rmsdxtol

      write(outfile,1000)
      write(outfile,1100)  maxg, maxgtol
      write(outfile,1200)  rmsg, rmsgtol
      write(outfile,1300)  maxdx, maxdxtol
      write(outfile,1400)  rmsdx, rmsdxtol
      write(outfile,1500)  

1000  format(/2x,'Convergence: ',/)
1100  format(2x,'Max Gradient     :',d10.3,t40,
     *          'Convergence Requirement  :',d10.3)
1200  format(2x,'RMS Gradient     :',d10.3,t40,
     *          'Convergence Requirement  :',d10.3)
1300  format(2x,'Max Displacement :',d10.3,t40,
     *          'Convergence Requirement  :',d10.3)
1400  format(2x,'RMS Displacement :',d10.3,t40,
     *          'Convergence Requirement  :',d10.3)
1500  format(/,80('-'),/)

      return
      end

